package com.mgh.pmdm.ejemplosui

import android.content.Intent
import android.os.Bundle
import android.view.View
import android.widget.Toast
import androidx.recyclerview.widget.LinearLayoutManager

import com.mgh.pmdm.ejemplosui.databinding.ActivityMainBinding


// La actividad principal, además de descender de Actividad con menús
// implementará la interfaz MiDialogFragment.OnOKOrCancelListener
class MainActivity : ActivityWithMenus(), MiDialogFragment.OnOKOrCancelListener {

    // Clase de vinculación
    private lateinit var binding: ActivityMainBinding

    // Propiedad privada para almacenar temporalmente un item
    // se utiliza cuando se va a eliminar un parque
    private var itemToRemove:Park?

    init{
        itemToRemove=null
    }

    /*********************************************
     * Métodos del ciclo de vida de la actividad
     ********************************************/

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        // Preparamos la clase de vinculación y establecemos el
        // contenido de la pantalla principal
        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)

        // Poblamos la lista de parques
        Parks.populate(this, R.raw.parks)



        // Preparació del RecyclerView:

        // 1. Asociamos el LayoutManager
        // Modificaciones para el PP5
        //binding.ParksRecyclerView.layoutManager = LinearLayoutManager(this)
        binding.ParksRecyclerView.layoutManager = LinearLayoutManager(this, LinearLayoutManager.HORIZONTAL,false)

        // 2. Determinamos que tiene tamaño fijo (optimización)
        binding.ParksRecyclerView.setHasFixedSize(true)
        // 3. Asignamos el adaptador al RecyclerView, proporcionándole
        //    dos funciones lambda, que invocarán los callbacks definidos en esta clase,
        //    para gestionar el click y el click largo sobre un item del RecyclerView.
        binding.ParksRecyclerView.adapter = AdaptadorParques(
            { park: Park, v: View -> itemClicked(park, v) },
            { park: Park, v: View -> itemLongClicked(park, v) }
        )
    }


    // Este método se lanza en el ciclo de vida siempre que la actividad vuelve al
    // primer plano. Cuando editemos o añadamos un parque nuevo, y hagamos click en el
    // botón de Atrás en la actividad EditParkArctivity, se invoca este método
    override fun onResume() {
        super.onResume()
        // Cuando la vista vuelve a primer plano, redibujamos el RecyclerView
        // Por si se hubo modificaciones en el contenido.
        binding.ParksRecyclerView.adapter?.notifyDataSetChanged()
    }

    /*************************************************************************
     * Métodos de callback para cuando se hace click o click largo en un item
     **************************************************************************/

    private fun itemClicked(park: Park, v: View) {
        // Creamos un nuevo Intent para abrir la actividad de edición
        // proporcionándole el parque sobre el que se ha hecho click

        val intent = Intent(this, EditParkActivity::class.java).apply{
            // Y le proporcionamos el parque, ya que este es serializable
            putExtra("com.mgh.pmdm.ejemplosui.EditPark.Park", park)
        }
        startActivity(intent)
    }

    private fun itemLongClicked(park: Park, v: View): Boolean {
        // Con el click largo eliminamos el parque de la vista

        // Establecemos el elemento a eliminar
        itemToRemove=park

        // e Invocamos al diálogo para preguntar si deseamos eliminar el registro
        val miDialogo = MiDialogFragment(
            resources.getString(R.string.deletePark),
            resources.getString(R.string.askDeletePark)+park.name+"?")
        miDialogo.show(supportFragmentManager, "miDialogo")
        return true

    }

    // Métodos para implementar la interfaz MiDialogFragment.OnOKOrCancelListener
    override fun onPositiveClick() {
        // Si se pulsa en Aceptar en el diálogo
        // eliminamos el item.
        itemToRemove?.also {
            val index=Parks.remove(it)

            // Y actualizamos el adaptador del RecyclerView
            // En lugar de DataSetChanged, utilizamos el método notifyItemRemoved
            // indicando la posición eliminada. De este modo, se aplica una
            // animación en el borrado.
            binding.ParksRecyclerView.adapter?.notifyItemRemoved(index)
        }
    }

    override fun onCancelClick() {
        // Si pulsa cancelar mostramos un toast
        Toast.makeText(
            applicationContext,
            resources.getString(R.string.actionCancelled), Toast.LENGTH_SHORT
        ).show()
    }

}